@@ -16,24 +16,31 @@
 Handles Jingle RTP sessions (XEP 0167)
 """
 
+import os
 import logging
 import socket
+from collections import deque
+from datetime import datetime
+
 import nbxmpp
-import gi
-from gi.repository import Farstream
+from nbxmpp.namespaces import Namespace
 
-gi.require_version('Gst', '1.0')
-from gi.repository import Gst
 from gi.repository import GLib
 
+try:
+    from gi.repository import Farstream
+    from gi.repository import Gst
+except Exception:
+    pass
+
 from gajim.common import app
 from gajim.common.i18n import _
 from gajim.common.jingle_transport import JingleTransportICEUDP
-from gajim.common.jingle_content import contents, JingleContent, JingleContentSetupException
-from gajim.common.connection_handlers_events import InformationEvent
+from gajim.common.jingle_content import contents
+from gajim.common.jingle_content import JingleContent
+from gajim.common.jingle_content import JingleContentSetupException
 from gajim.common.jingle_session import FailedApplication
 
-from collections import deque
 
 log = logging.getLogger('gajim.c.jingle_rtp')
 
@@ -89,8 +96,8 @@ class JingleRTPContent(JingleContent):
         # due to bad controlling-mode
 
         params = {'controlling-mode': self.session.weinitiate, 'debug': False}
-        if app.config.get('use_stun_server'):
-            stun_server = app.config.get('stun_server')
+        if app.settings.get('use_stun_server'):
+            stun_server = app.settings.get('stun_server')
             if not stun_server and self.session.connection._stun_servers:
                 stun_server = self.session.connection._stun_servers[0]['host']
             if stun_server:
@@ -111,25 +118,21 @@ class JingleRTPContent(JingleContent):
         return JingleContent.is_ready(self) and self.candidates_ready
 
     def make_bin_from_config(self, config_key, pipeline, text):
-        pipeline = pipeline % app.config.get(config_key)
+        pipeline = pipeline % app.settings.get(config_key)
+        log.debug('Pipeline: %s', str(pipeline))
         try:
             gst_bin = Gst.parse_bin_from_description(pipeline, True)
             return gst_bin
-        except GLib.GError as e:
-            app.nec.push_incoming_event(
-                InformationEvent(
-                    None, conn=self.session.connection, level='error',
-                    pri_txt=_('%s configuration error') % text.capitalize(),
-                    sec_txt=_('Couldn’t set up %(text)s. Check your '
-                    'configuration.\n\nPipeline was:\n%(pipeline)s\n\n'
-                    'Error was:\n%(error)s') % {'text': text,
-                    'pipeline': pipeline, 'error': str(e)}))
+        except GLib.Error as err:
+            log.error('Couldn’t set up %s. Check your '
+                      'configuration. Pipeline: %s'
+                      'Error: %s', text, pipeline, err)
             raise JingleContentSetupException
 
     def add_remote_candidates(self, candidates):
         JingleContent.add_remote_candidates(self, candidates)
-        # FIXME: connectivity should not be etablished yet
-        # Instead, it should be etablished after session-accept!
+        # FIXME: connectivity should not be established yet
+        # Instead, it should be established after session-accept!
         if self.sent:
             self.p2pstream.add_remote_candidates(candidates)
 
@@ -141,30 +144,31 @@ class JingleRTPContent(JingleContent):
             raise Exception("There is a DTMF batch already running")
         events = deque(events)
         self._dtmf_running = True
-        self._start_dtmf(events.popleft())
+        self.start_dtmf(events.popleft())
         GLib.timeout_add(500, self._next_dtmf, events)
 
     def _next_dtmf(self, events):
-        self._stop_dtmf()
+        self.stop_dtmf()
         if events:
-            self._start_dtmf(events.popleft())
+            self.start_dtmf(events.popleft())
             GLib.timeout_add(500, self._next_dtmf, events)
         else:
             self._dtmf_running = False
 
-    def _start_dtmf(self, event):
-        if event in ('*', '#'):
-            event = {'*': Farstream.DTMFEvent.STAR,
-                     '#': Farstream.DTMFEvent.POUND}[event]
+    def start_dtmf(self, key: str) -> None:
+        if key == 'star':
+            event = Farstream.DTMFEvent.STAR
+        elif key == 'pound':
+            event = Farstream.DTMFEvent.POUND
         else:
-            event = int(event)
+            event = int(key)
         self.p2psession.start_telephony_event(event, 2)
 
-    def _stop_dtmf(self):
+    def stop_dtmf(self):
         self.p2psession.stop_telephony_event()
 
     def _fill_content(self, content):
-        content.addChild(nbxmpp.NS_JINGLE_RTP + ' description',
+        content.addChild(Namespace.JINGLE_RTP + ' description',
                          attrs={'media': self.media},
                          payload=list(self.iter_codecs()))
 
@@ -175,15 +179,17 @@ class JingleRTPContent(JingleContent):
         self.sink.set_state(Gst.State.PLAYING)
         self.funnel.set_state(Gst.State.PLAYING)
 
-    def _on_src_pad_added(self, stream, pad, codec):
+    def _on_src_pad_added(self, _stream, pad, codec):
+        log.info('Used codec: %s', codec.to_string())
         if not self.funnel:
             self._setup_funnel()
         pad.link(self.funnel.get_request_pad('sink_%u'))
 
-    def _on_gst_message(self, bus, message):
+    def _on_gst_message(self, _bus, message):
         if message.type == Gst.MessageType.ELEMENT:
             name = message.get_structure().get_name()
-            log.debug('gst element message: %s: %s', name, message)
+            message_string = message.get_structure().to_string()
+            log.debug('gst element message: %s', message_string)
             if name == 'farstream-new-active-candidate-pair':
                 pass
             elif name == 'farstream-recv-codecs-changed':
@@ -220,17 +226,15 @@ class JingleRTPContent(JingleContent):
                 log.error('Farstream error #%d!\nMessage: %s',
                           message.get_structure().get_value('error-no'),
                           message.get_structure().get_value('error-msg'))
+
         elif message.type == Gst.MessageType.ERROR:
             # TODO: Fix it to fallback to videotestsrc anytime an error occur,
             # or raise an error, Jingle way
             # or maybe one-sided stream?
-            if not self.stream_failed_once:
-                app.nec.push_incoming_event(
-                    InformationEvent(
-                        None, dialog_name='gstreamer-error',
-                        kwargs={'error': message.get_structure().get_value('gerror'),
-                                'debug': message.get_structure().get_value('debug')}))
-
+            gerror_msg = message.get_structure().get_value('gerror')
+            debug_msg = message.get_structure().get_value('debug')
+            log.error(gerror_msg)
+            log.error(debug_msg)
             sink_pad = self.p2psession.get_property('sink-pad')
 
             # Remove old source
@@ -269,26 +273,30 @@ class JingleRTPContent(JingleContent):
 #                        Farstream.StreamDirection.BOTH)
         JingleContent.on_negotiated(self)
 
-    def __on_remote_codecs(self, stanza, content, error, action):
+    def __on_remote_codecs(self, _stanza, content, _error, _action):
         """
         Get peer codecs from what we get from peer
         """
-
         codecs = []
         for codec in content.getTag('description').iterTags('payload-type'):
             if not codec['id'] or not codec['name'] or not codec['clockrate']:
                 # ignore invalid payload-types
                 continue
-            c = Farstream.Codec.new(int(codec['id']), codec['name'],
-                                    self.farstream_media, int(codec['clockrate']))
+            farstream_codec = Farstream.Codec.new(
+                int(codec['id']),
+                codec['name'],
+                self.farstream_media,
+                int(codec['clockrate']))
             if 'channels' in codec:
-                c.channels = int(codec['channels'])
+                farstream_codec.channels = int(codec['channels'])
             else:
-                c.channels = 1
-            for p in codec.iterTags('parameter'):
-                c.add_optional_parameter(p['name'], str(p['value']))
-            codecs.append(c)
-
+                farstream_codec.channels = 1
+            for param in codec.iterTags('parameter'):
+                farstream_codec.add_optional_parameter(
+                    param['name'], str(param['value']))
+            log.debug('Remote codec: %s (%s)',
+                      codec['name'], codec['clockrate'])
+            codecs.append(farstream_codec)
         if codecs:
             try:
                 self.p2pstream.set_remote_codecs(codecs)
@@ -301,8 +309,9 @@ class JingleRTPContent(JingleContent):
             attrs = {
                 'name': codec.encoding_name,
                 'id': codec.id,
-                'channels': codec.channels
             }
+            if codec.channels > 0:
+                attrs['channels'] = codec.channels
             if codec.clock_rate:
                 attrs['clockrate'] = codec.clock_rate
             if codec.optional_params:
@@ -335,43 +344,74 @@ class JingleAudio(JingleRTPContent):
         JingleRTPContent.__init__(self, session, 'audio', transport)
         self.setup_stream()
 
-    def set_mic_volume(self, vol):
+    def set_mic_volume(self, vol: float) -> None:
         """
-        vol must be between 0 ans 1
+        vol must be between 0 and 1
         """
         self.mic_volume.set_property('volume', vol)
 
-    def set_out_volume(self, vol):
+    def set_out_volume(self, vol: float) -> None:
         """
-        vol must be between 0 ans 1
+        vol must be between 0 and 1
         """
         self.out_volume.set_property('volume', vol)
 
     def setup_stream(self):
         JingleRTPContent.setup_stream(self, self._on_src_pad_added)
 
-        # Configure SPEEX
-        # Workaround for psi (not needed since rev
-        # 147aedcea39b43402fe64c533d1866a25449888a):
-        #  place 16kHz before 8kHz, as buggy psi versions will take in
-        #  account only the first codec
-
-        codecs = [
+        # list of codecs that are explicitly allowed
+        allow_codecs = [
+            Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'OPUS',
+                                Farstream.MediaType.AUDIO, 48000),
+            Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'SPEEX',
+                                Farstream.MediaType.AUDIO, 32000),
+            Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'G722',
+                                Farstream.MediaType.AUDIO, 8000),
             Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'SPEEX',
                                 Farstream.MediaType.AUDIO, 16000),
+            Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'PCMA',
+                                Farstream.MediaType.AUDIO, 8000),
+            Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'PCMU',
+                                Farstream.MediaType.AUDIO, 8000),
             Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'SPEEX',
-                                Farstream.MediaType.AUDIO, 8000)]
-        self.p2psession.set_codec_preferences(codecs)
+                                Farstream.MediaType.AUDIO, 8000),
+            Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'AMR',
+                                Farstream.MediaType.AUDIO, 8000),
+            ]
+
+        # disable all other codecs
+        disable_codecs = []
+        codecs_without_config = self.p2psession.props.codecs_without_config
+        allowed_encoding_names = [c.encoding_name for c in allow_codecs]
+        allowed_encoding_names.append('telephone-event')
+        for codec in codecs_without_config:
+            if codec.encoding_name not in allowed_encoding_names:
+                disable_codecs.append(Farstream.Codec.new(
+                    Farstream.CODEC_ID_DISABLE,
+                    codec.encoding_name,
+                    Farstream.MediaType.AUDIO,
+                    codec.clock_rate))
+
+        self.p2psession.set_codec_preferences(allow_codecs + disable_codecs)
 
         # the local parts
         # TODO: Add queues?
-        self.src_bin = self.make_bin_from_config('audio_input_device',
-                                                 '%s ! audioconvert',
-                                                 _("audio input"))
+        self.src_bin = self.make_bin_from_config(
+            'audio_input_device',
+            '''%s
+            ! webrtcdsp
+                echo-suppression-level=high
+                noise-suppression-level=very-high
+                voice-detection=true
+            ! audioconvert ''',
+            _('audio input'))
 
-        self.sink = self.make_bin_from_config('audio_output_device',
-                                              'audioconvert ! volume name=gajim_out_vol ! %s',
-                                              _("audio output"))
+        # setting name=webrtcechoprobe0 is needed because lingering probes
+        # cause a bug in subsequent calls
+        self.sink = self.make_bin_from_config(
+            'audio_output_device',
+            'audioconvert ! volume name=gajim_out_vol ! webrtcechoprobe name=webrtcechoprobe0 ! %s',
+            _('audio output'))
 
         self.mic_volume = self.src_bin.get_by_name('gajim_vol')
         self.out_volume = self.sink.get_by_name('gajim_out_vol')
@@ -380,19 +420,17 @@ class JingleAudio(JingleRTPContent):
         self.pipeline.add(self.sink)
         self.pipeline.add(self.src_bin)
 
-        self.src_bin.get_static_pad('src').link(self.p2psession.get_property(
-            'sink-pad'))
+        self.src_bin.get_static_pad('src').link(
+            self.p2psession.get_property('sink-pad'))
 
         # The following is needed for farstream to process ICE requests:
         self.pipeline.set_state(Gst.State.PLAYING)
 
 
 class JingleVideo(JingleRTPContent):
-    def __init__(self, session, transport=None, in_xid=0, out_xid=0):
+    def __init__(self, session, transport=None):
         JingleRTPContent.__init__(self, session, 'video', transport)
-        self.in_xid = in_xid
-        self.out_xid = out_xid
-        self.out_xid_set = False
+        self.sink = None
         self.setup_stream()
 
     def setup_stream(self):
@@ -402,70 +440,82 @@ class JingleVideo(JingleRTPContent):
         JingleRTPContent.setup_stream(self, self._on_src_pad_added)
         bus = self.pipeline.get_bus()
         bus.enable_sync_message_emission()
-        bus.connect('sync-message::element', self._on_sync_message)
 
-        # the local parts
-        if app.config.get('video_framerate'):
-            framerate = 'videorate ! video/x-raw,framerate=%s ! ' % \
-                app.config.get('video_framerate')
-        else:
-            framerate = ''
-        try:
-            w, h = app.config.get('video_size').split('x')
-        except Exception:
-            w = h = None
-        if w and h:
-            video_size = 'video/x-raw,width=%s,height=%s ! ' % (w, h)
-        else:
-            video_size = ''
-        if app.config.get('video_see_self'):
-            tee = '! tee name=t ! queue ! videoscale ! ' + \
-                'video/x-raw,width=160,height=120 ! videoconvert ! ' + \
-                '%s t. ! queue ' % app.config.get(
-                    'video_output_device')
+        # list of codecs that are explicitly allowed
+        # for now only VP8/H264 (available in gst-plugins-good)
+        allow_codecs = [
+            #Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'VP9',
+            #                    Farstream.MediaType.VIDEO, 90000),
+            Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'VP8',
+                                Farstream.MediaType.VIDEO, 90000),
+            Farstream.Codec.new(Farstream.CODEC_ID_ANY, 'H264',
+                                Farstream.MediaType.VIDEO, 90000),
+        ]
+
+        # disable all other codecs
+        disable_codecs = []
+        codecs_without_config = self.p2psession.props.codecs_without_config
+        allowed_encoding_names = [c.encoding_name for c in allow_codecs]
+        for codec in codecs_without_config:
+            if codec.encoding_name not in allowed_encoding_names:
+                disable_codecs.append(Farstream.Codec.new(
+                    Farstream.CODEC_ID_DISABLE,
+                    codec.encoding_name,
+                    Farstream.MediaType.VIDEO,
+                    codec.clock_rate))
+
+        self.p2psession.set_codec_preferences(allow_codecs + disable_codecs)
+
+    def do_setup(self, self_display_sink, other_sink):
+        if app.settings.get('video_see_self'):
+            tee = ('! tee name=split ! queue name=self-display-queue split. ! '
+                   'queue name=network-queue')
         else:
             tee = ''
 
-        self.src_bin = self.make_bin_from_config('video_input_device',
-                                                 '%%s %s! %svideoscale ! %svideoconvert' %
-                                                 (tee, framerate, video_size),
-                                                 _("video input"))
-
-        self.pipeline.add(self.src_bin)
-        self.pipeline.set_state(Gst.State.PLAYING)
-
-        self.sink = self.make_bin_from_config('video_output_device',
-                                              'videoscale ! videoconvert ! %s',
-                                              _("video output"))
-
+        self.sink = other_sink
         self.pipeline.add(self.sink)
 
-        self.src_bin.get_static_pad('src').link(self.p2psession.get_property(
-            'sink-pad'))
+        self.src_bin = self.make_bin_from_config(
+            'video_input_device',
+            '%%s %s' % tee,
+            _('video input'))
+
+        self.pipeline.add(self.src_bin)
+        if app.settings.get('video_see_self'):
+            self.pipeline.add(self_display_sink)
+            self_display_queue = self.src_bin.get_by_name('self-display-queue')
+            self_display_queue.get_static_pad('src').link_maybe_ghosting(
+                self_display_sink.get_static_pad('sink'))
+
+        self.src_bin.get_static_pad('src').link(
+            self.p2psession.get_property('sink-pad'))
 
         # The following is needed for farstream to process ICE requests:
         self.pipeline.set_state(Gst.State.PLAYING)
 
-    def _on_sync_message(self, bus, message):
-        if message.get_structure() is None:
-            return False
-        if message.get_structure().get_name() == 'prepare-window-handle':
-            message.src.set_property('force-aspect-ratio', True)
-            imagesink = message.src
-            if app.config.get('video_see_self') and not self.out_xid_set:
-                imagesink.set_window_handle(self.out_xid)
-                self.out_xid_set = True
-            else:
-                imagesink.set_window_handle(self.in_xid)
+        if log.getEffectiveLevel() == logging.DEBUG:
+            # Use 'export GST_DEBUG_DUMP_DOT_DIR=/tmp/' before starting Gajim
+            timestamp = datetime.now().strftime('%m-%d-%Y-%H-%M-%S')
+            name = f'video-graph-{timestamp}'
+            debug_dir = os.environ.get('GST_DEBUG_DUMP_DOT_DIR')
+            name_dot = f'{debug_dir}/{name}.dot'
+            name_png = f'{debug_dir}/{name}.png'
+            Gst.debug_bin_to_dot_file(
+                self.pipeline, Gst.DebugGraphDetails.ALL, name)
+            if debug_dir:
+                try:
+                    os.system(f'dot -Tpng {name_dot} > {name_png}')
+                except Exception:
+                    log.debug('Could not save pipeline graph. Make sure '
+                              'graphviz is installed.')
 
     def get_fallback_src(self):
         # TODO: Use avatar?
-        pipeline = 'videotestsrc is-live=true ! video/x-raw,framerate=10/1 ! videoconvert'
+        pipeline = ('videotestsrc is-live=true ! video/x-raw,framerate=10/1 ! '
+                    'videoconvert')
         return Gst.parse_bin_from_description(pipeline, True)
 
-    def destroy(self):
-        JingleRTPContent.destroy(self)
-        self.pipeline.get_bus().disconnect_by_func(self._on_sync_message)
 
 def get_content(desc):
     if desc['media'] == 'audio':
@@ -473,4 +523,5 @@ def get_content(desc):
     if desc['media'] == 'video':
         return JingleVideo
 
-contents[nbxmpp.NS_JINGLE_RTP] = get_content
+
+contents[Namespace.JINGLE_RTP] = get_content
