@@ -14,24 +14,31 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-import logging
 import os
 import sys
+import time
+import logging
+from datetime import datetime
+from typing import Optional
 
+from gajim.common import app
+from gajim.common import configpaths
 from gajim.common.i18n import _
 
-def parseLogLevel(arg):
+
+def parseLogLevel(arg: str) -> int:
     """
-    Eiter numeric value or level name from logging module
+    Either numeric value or level name from logging module
     """
     if arg.isdigit():
         return int(arg)
     if arg.isupper() and hasattr(logging, arg):
         return getattr(logging, arg)
-    print(_('%s is not a valid loglevel') % repr(arg))
+    print(_('%s is not a valid loglevel') % repr(arg), file=sys.stderr)
     return 0
 
-def parseLogTarget(arg):
+
+def parseLogTarget(arg: str) -> str:
     """
     [gajim.]c.x.y  ->  gajim.c.x.y
     .other_logger  ->  other_logger
@@ -46,7 +53,8 @@ def parseLogTarget(arg):
         return arg
     return 'gajim.' + arg
 
-def parseAndSetLogLevels(arg):
+
+def parseAndSetLogLevels(arg: str) -> None:
     """
     [=]LOGLEVEL     ->  gajim=LOGLEVEL
     gajim=LOGLEVEL  ->  gajim=LOGLEVEL
@@ -69,11 +77,11 @@ def parseAndSetLogLevels(arg):
             target = parseLogTarget(target.strip())
             if target:
                 logging.getLogger(target).setLevel(level)
-                print("Logger %s level set to %d" % (target, level))
+                print("Logger %s level set to %d" % (target, level),
+                      file=sys.stderr)
 
 
 class colors:
-    # pylint: disable=C0326
     NONE         = chr(27) + "[0m"
     BLACk        = chr(27) + "[30m"
     RED          = chr(27) + "[31m"
@@ -92,9 +100,11 @@ class colors:
     BRIGHT_CYAN  = chr(27) + "[36;1m"
     WHITE        = chr(27) + "[37;1m"
 
-def colorize(text, color):
+
+def colorize(text: str, color: str) -> str:
     return color + text + colors.NONE
 
+
 class FancyFormatter(logging.Formatter):
     """
     An eye-candy formatter with colors
@@ -107,34 +117,44 @@ class FancyFormatter(logging.Formatter):
         'CRITICAL': colors.BRIGHT_RED,
     }
 
-    def __init__(self, fmt, datefmt=None, use_color=False):
+    def __init__(self,
+                 fmt: Optional[str] = None,
+                 datefmt: Optional[str] = None,
+                 use_color: bool = False) -> None:
         logging.Formatter.__init__(self, fmt, datefmt)
         self.use_color = use_color
 
-    def formatTime(self, record, datefmt=None):
+    def formatTime(self,
+                   record: logging.LogRecord,
+                   datefmt: Optional[str] = None) -> str:
         f = logging.Formatter.formatTime(self, record, datefmt)
         if self.use_color:
             f = colorize(f, colors.DARK_GRAY)
         return f
 
-    def format(self, record):
+    def format(self, record: logging.LogRecord) -> str:
         level = record.levelname
         record.levelname = '(%s)' % level[0]
 
         if self.use_color:
             c = FancyFormatter.colors_mapping.get(level, '')
             record.levelname = colorize(record.levelname, c)
-            record.name = colorize(record.name, colors.CYAN)
+            record.name = '%-25s' % colorize(record.name, colors.CYAN)
         else:
-            record.name += ':'
+            record.name = '%-25s|' % record.name
 
         return logging.Formatter.format(self, record)
 
 
-def init():
+def init() -> None:
     """
     Iinitialize the logging system
     """
+
+    if app.get_debug_mode():
+        _cleanup_debug_logs()
+        _redirect_output()
+
     use_color = False
     if os.name != 'nt':
         use_color = sys.stderr.isatty()
@@ -142,36 +162,67 @@ def init():
     consoleloghandler = logging.StreamHandler()
     consoleloghandler.setFormatter(
         FancyFormatter(
-            '%(asctime)s %(levelname)s %(name)s %(message)s',
+            '%(asctime)s %(levelname)s %(name)-35s %(message)s',
             '%x %H:%M:%S',
             use_color
         )
     )
 
-    # fake the root logger so we have 'gajim' root name instead of 'root'
     root_log = logging.getLogger('gajim')
     root_log.setLevel(logging.WARNING)
     root_log.addHandler(consoleloghandler)
     root_log.propagate = False
 
-    # handle nbxmpp logs too
     root_log = logging.getLogger('nbxmpp')
+    root_log.setLevel(logging.ERROR)
+    root_log.addHandler(consoleloghandler)
+    root_log.propagate = False
+
+    root_log = logging.getLogger('gnupg')
     root_log.setLevel(logging.WARNING)
     root_log.addHandler(consoleloghandler)
     root_log.propagate = False
 
-def set_loglevels(loglevels_string):
+    # GAJIM_DEBUG is set only on Windows when using Gajim-Debug.exe
+    # Gajim-Debug.exe shows a command line prompt and we want to redirect
+    # log output to it
+    if app.get_debug_mode() or os.environ.get('GAJIM_DEBUG', False):
+        set_verbose()
+
+
+def set_loglevels(loglevels_string: str) -> None:
     parseAndSetLogLevels(loglevels_string)
 
-def set_verbose():
+
+def set_verbose() -> None:
     parseAndSetLogLevels('gajim=DEBUG')
     parseAndSetLogLevels('.nbxmpp=INFO')
 
-def set_quiet():
+
+def set_quiet() -> None:
     parseAndSetLogLevels('gajim=CRITICAL')
     parseAndSetLogLevels('.nbxmpp=CRITICAL')
 
 
+def _redirect_output() -> None:
+    debug_folder = configpaths.get('DEBUG')
+    date = datetime.today().strftime('%d%m%Y-%H%M%S')
+    filename = '%s-debug.log' % date
+    fd = open(debug_folder / filename, 'a', encoding='utf8')
+    sys.stderr = sys.stdout = fd
+
+
+def _cleanup_debug_logs() -> None:
+    debug_folder = configpaths.get('DEBUG')
+    debug_files = list(debug_folder.glob('*-debug.log*'))
+    now = time.time()
+    for file in debug_files:
+        # Delete everything older than 3 days
+        if file.stat().st_ctime < now - 259200:
+            file.unlink()
+
+
+
 # tests
 if __name__ == '__main__':
     init()
