@@ -23,7 +23,7 @@ import struct
 import hashlib
 import os
 import time
-import platform
+import sys
 import logging
 from errno import EWOULDBLOCK
 from errno import ENOBUFS
@@ -85,7 +85,7 @@ class SocksQueue:
         log.debug('Start listening for socks5 connection')
         sid = file_props.sid
         self.sha_handlers[sha_str] = (sha_handler, sid)
-        if self.listener is None or self.listener.connections == []:
+        if self.listener is None or not self.listener.connections:
             self.listener = Socks5Listener(self.idlequeue, port, file_props,
                 fingerprint=fingerprint)
             self.listener.queue = self
@@ -296,7 +296,7 @@ class SocksQueue:
     def activate_proxy(self, idx):
         if not self.isHashInSockObjs(self.senders, idx):
             return
-        for key in self.senders:
+        for key in list(self.senders):
             if idx in key:
                 sender = self.senders[key]
                 if sender.file_props.type_ != 's':
@@ -333,7 +333,7 @@ class SocksQueue:
 
     def isHashInSockObjs(self, sockobjs, hash_):
         '''
-        It tells wether there is a particular hash in sockobjs or not
+        It tells whether there is a particular hash in sockobjs or not
         '''
         for key in sockobjs:
             if hash_ in key:
@@ -683,17 +683,20 @@ class Socks5:
             lenn = 0
             try:
                 lenn = self._send(buff)
-            except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-            OpenSSL.SSL.WantX509LookupError) as e:
-                log.info('SSL rehandshake request: %s', repr(e))
-                raise e
-            except Exception as e:
-                if e.errno not in (EINTR, ENOBUFS, EWOULDBLOCK):
-                    # peer stopped reading
-                    self.state = 8 # end connection
-                    self.disconnect()
-                    self.file_props.error = -1
-                    return -1
+            except (OpenSSL.SSL.WantReadError,
+                    OpenSSL.SSL.WantWriteError,
+                    OpenSSL.SSL.WantX509LookupError) as err:
+                log.info('SSL rehandshake request: %s', repr(err))
+                raise err
+            except OpenSSL.SSL.SysCallError:
+                return self._on_send_exception()
+            except socket.error as err:
+                if err.errno not in (EINTR, ENOBUFS, EWOULDBLOCK):
+                    return self._on_send_exception()
+            except Exception as err:
+                log.error(err)
+                return self._on_send_exception()
+
             self.size += lenn
             current_time = time.time()
             self.file_props.elapsed_time += current_time - \
@@ -714,10 +717,17 @@ class Socks5:
                 return None
             self.file_props.stalled = False
             return lenn
-        else:
-            self.state = 8 # end connection
-            self.disconnect()
-            return -1
+
+        self.state = 8 # end connection
+        self.disconnect()
+        return -1
+
+    def _on_send_exception(self):
+        # peer stopped reading
+        self.state = 8 # end connection
+        self.disconnect()
+        self.file_props.error = -1
+        return -1
 
     def get_file_contents(self, timeout):
         """
@@ -1292,8 +1302,7 @@ class Socks5Client(Socks5):
                     return
                 if self.state < 5:
                     self.idlequeue.set_read_timeout(self.fd, CONNECT_TIMEOUT)
-                    result = self.main(0)
-                    self.queue.process_result(result, self)
+                    self.main(0)
                 elif self.state == 5: # wait for proxy reply
                     pass
                 elif self.file_props.type_ == 'r':
@@ -1390,7 +1399,7 @@ class Socks5ReceiverServer(Socks5Server, Socks5Receiver):
 class Socks5Listener(IdleObject):
     def __init__(self, idlequeue, port, fp, fingerprint=None):
         """
-        Handle all incomming connections on (0.0.0.0, port)
+        Handle all incoming connections on (0.0.0.0, port)
 
         This class implements IdleObject, but we will expect
         only pollin events though
@@ -1431,7 +1440,7 @@ class Socks5Listener(IdleObject):
             # Under windows Vista, we need that to listen on ipv6 AND ipv4
             # Doesn't work under windows XP
             if os.name == 'nt':
-                if int(platform.win32_ver()[0]) >= 6:  # Win Vista +
+                if sys.getwindowsversion().major >= 6:  # Win Vista +
                     # 47 is socket.IPPROTO_IPV6
                     # 27 is socket.IPV6_V6ONLY under windows, but not defined ...
                     self._serv.setsockopt(41, 27, 0)
@@ -1460,7 +1469,7 @@ class Socks5Listener(IdleObject):
 
     def pollin(self):
         """
-        Accept a new incomming connection and notify queue
+        Accept a new incoming connection and notify queue
         """
         sock = self.accept_conn()
         self.queue.on_connection_accepted(sock, self)
@@ -1481,7 +1490,7 @@ class Socks5Listener(IdleObject):
 
     def accept_conn(self):
         """
-        Accept a new incomming connection
+        Accept a new incoming connection
         """
         _sock = self._serv.accept()
         _sock[0].setblocking(False)
