@@ -12,13 +12,19 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-import os
+from typing import Any
+from typing import List
+from typing import Optional
+from typing import Tuple
+from typing import Union
+
 import logging
 
 from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import Gtk
 from gi.repository import Gio
+from gi.repository import GObject
 
 from nbxmpp.client import Client
 from nbxmpp.protocol import JID
@@ -27,12 +33,15 @@ from nbxmpp.const import Mode
 from nbxmpp.const import StreamError
 from nbxmpp.const import ConnectionProtocol
 from nbxmpp.const import ConnectionType
-from nbxmpp.util import is_error_result
+from nbxmpp.errors import StanzaError
+from nbxmpp.errors import MalformedStanzaError
+from nbxmpp.errors import RegisterStanzaError
+from nbxmpp.task import Task
 
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import helpers
-from gajim.common.nec import NetworkEvent
+from gajim.common.events import StanzaReceived, StanzaSent
 from gajim.common.helpers import open_uri
 from gajim.common.helpers import validate_jid
 from gajim.common.helpers import get_proxy
@@ -40,23 +49,24 @@ from gajim.common.i18n import _
 from gajim.common.const import SASL_ERRORS
 from gajim.common.const import GIO_TLS_ERRORS
 
-from gajim.gtk.assistant import Assistant
-from gajim.gtk.assistant import Page
-from gajim.gtk.assistant import SuccessPage
-from gajim.gtk.dataform import DataFormWidget
-from gajim.gtk.util import get_builder
-from gajim.gtk.util import open_window
-from gajim.gtk.util import ensure_not_destroyed
+from .assistant import Assistant
+from .assistant import Page
+from .assistant import SuccessPage
+from .assistant import ErrorPage
+from .dataform import DataFormWidget
+from .builder import get_builder
+from .util import open_window
+from .util import get_color_for_account
+from .util import get_app_window
 
-
-log = logging.getLogger('gajim.gtk.account_wizard')
+log = logging.getLogger('gajim.gui.account_wizard')
 
 
 class AccountWizard(Assistant):
-    def __init__(self):
+    def __init__(self) -> None:
         Assistant.__init__(self, height=500)
 
-        self._destroyed = False
+        self._destroyed: bool = False
 
         self.add_button('signup', _('Sign Up'), complete=True,
                         css_class='suggested-action')
@@ -66,22 +76,20 @@ class AccountWizard(Assistant):
                         css_class='suggested-action')
         self.add_button('back', _('Back'))
 
-        self.add_pages({'login': Login(self._on_button_clicked),
+        self.add_pages({'login': Login(),
                         'signup': Signup(),
                         'advanced': AdvancedSettings(),
                         'security-warning': SecurityWarning(),
                         'form': Form(),
                         'redirect': Redirect(),
                         'success': Success(),
+                        'error': Error(),
                         })
 
         self._progress = self.add_default_page('progress')
-        self._error = self.add_default_page('error')
-        self._error.set_heading(_('An error occurred during account creation'))
 
-        self.set_button_visible_func(self._visible_func)
-
-        self.connect('button-clicked', self._on_button_clicked)
+        self.get_page('login').connect('clicked', self._on_button_clicked)
+        self.connect('button-clicked', self._on_assistant_button_clicked)
         self.connect('page-changed', self._on_page_changed)
         self.connect('destroy', self._on_destroy)
 
@@ -89,39 +97,31 @@ class AccountWizard(Assistant):
 
         self.update_proxy_list()
 
-        self._client = None
-        self._method = 'login'
+        self._client: Optional[Client] = None
+        self._method: str = 'login'
 
-    def _visible_func(self, _assistant, page_name):
-        if page_name in ('signup', 'form'):
-            return ['back', 'signup']
+    def get_currenct_method(self) -> str:
+        return self._method
 
-        if page_name in ('security-warning', 'advanced'):
-            return ['back', self._method]
+    def _on_button_clicked(self, _page: Gtk.Widget, button_name: str) -> None:
+        if button_name == 'login':
+            if self.get_page('login').is_advanced():
+                self.show_page('advanced',
+                               Gtk.StackTransitionType.SLIDE_LEFT)
+            else:
+                self._test_credentials()
 
-        if page_name in ('login', 'progress'):
-            return None
+        elif button_name == 'signup':
+            self.show_page('signup', Gtk.StackTransitionType.SLIDE_LEFT)
 
-        if page_name in ('error', 'redirect'):
-            return ['back']
-
-        if page_name == 'success':
-            return ['connect']
-
-        raise ValueError('page %s unknown' % page_name)
-
-    def _on_button_clicked(self, _assistant, button_name):
+    def _on_assistant_button_clicked(self,
+                                     _assistant: Assistant,
+                                     button_name: str
+                                     ) -> None:
         page = self.get_current_page()
 
         if button_name == 'login':
-            if page == 'login':
-                if self.get_page('login').is_advanced():
-                    self.show_page('advanced',
-                                   Gtk.StackTransitionType.SLIDE_LEFT)
-                else:
-                    self._test_credentials()
-
-            elif page == 'advanced':
+            if page == 'advanced':
                 self._test_credentials()
 
             elif page == 'security-warning':
@@ -131,10 +131,7 @@ class AccountWizard(Assistant):
                 self._test_credentials(ignore_all_errors=True)
 
         elif button_name == 'signup':
-            if page == 'login':
-                self.show_page('signup', Gtk.StackTransitionType.SLIDE_LEFT)
-
-            elif page == 'signup':
+            if page == 'signup':
                 if self.get_page('signup').is_advanced():
                     self.show_page('advanced',
                                    Gtk.StackTransitionType.SLIDE_LEFT)
@@ -193,41 +190,31 @@ class AccountWizard(Assistant):
             elif page == 'redirect':
                 self.show_page('login', Gtk.StackTransitionType.SLIDE_RIGHT)
 
-    def _on_page_changed(self, _assistant, page_name):
+    def _on_page_changed(self,
+                         _assistant: Assistant,
+                         page_name: str
+                         ) -> None:
         if page_name == 'signup':
             self._method = page_name
             self.get_page('signup').focus()
-            self.set_default_button('signup')
 
         elif page_name == 'login':
             self._method = page_name
             self.get_page('login').focus()
 
-        elif page_name == 'security-warning':
-            self.set_default_button('back')
-
-        elif page_name == 'advanced':
-            self.set_default_button(self._method)
-
-        elif page_name == 'success':
-            self.set_default_button('connect')
-
-        elif page_name in ('error', 'redirect'):
-            self.set_default_button('back')
-
         elif page_name == 'form':
             self.get_page('form').focus()
-            self.set_default_button('signup')
 
-    def update_proxy_list(self):
+    def update_proxy_list(self) -> None:
         self.get_page('advanced').update_proxy_list()
 
     def _get_base_client(self,
-                         domain,
-                         username,
-                         mode,
-                         advanced,
-                         ignore_all_errors):
+                         domain: str,
+                         username: str,
+                         mode: Mode,
+                         advanced: bool,
+                         ignore_all_errors: bool
+                         ) -> Client:
 
         client = Client(log_context='Account Wizard')
         client.set_domain(domain)
@@ -253,7 +240,7 @@ class AccountWizard(Assistant):
         client.subscribe('stanza-received', self._on_stanza_received)
         return client
 
-    def _disconnect(self):
+    def _disconnect(self) -> None:
         if self._client is None:
             return
         self._client.remove_subscriptions()
@@ -261,27 +248,31 @@ class AccountWizard(Assistant):
         self._client = None
 
     @staticmethod
-    def _on_stanza_sent(_client, _signal_name, stanza):
-        app.nec.push_incoming_event(NetworkEvent('stanza-sent',
-                                                 account='AccountWizard',
-                                                 stanza=stanza))
+    def _on_stanza_sent(_client: Client,
+                        _signal_name: str,
+                        stanza: Any
+                        ) -> None:
+        app.ged.raise_event(StanzaSent(account='AccountWizard',
+                                       stanza=stanza))
 
     @staticmethod
-    def _on_stanza_received(_client, _signal_name, stanza):
-        app.nec.push_incoming_event(NetworkEvent('stanza-received',
-                                                 account='AccountWizard',
-                                                 stanza=stanza))
+    def _on_stanza_received(_client: Client,
+                            _signal_name: str,
+                            stanza: Any
+                            ) -> None:
+        app.ged.raise_event(StanzaReceived(account='AccountWizard',
+                                           stanza=stanza))
 
-    def _test_credentials(self, ignore_all_errors=False):
+    def _test_credentials(self, ignore_all_errors: bool = False) -> None:
         self._show_progress_page(_('Connecting...'),
                                  _('Connecting to server...'))
         address, password = self.get_page('login').get_credentials()
-        jid = JID(address)
+        jid = JID.from_string(address)
         advanced = self.get_page('login').is_advanced()
 
         self._client = self._get_base_client(
-            jid.getDomain(),
-            jid.getNode(),
+            jid.domain,
+            jid.localpart,
             Mode.LOGIN_TEST,
             advanced,
             ignore_all_errors)
@@ -291,7 +282,7 @@ class AccountWizard(Assistant):
 
         self._client.connect()
 
-    def _test_anonymous_server(self, ignore_all_errors=False):
+    def _test_anonymous_server(self, ignore_all_errors: bool = False) -> None:
         self._show_progress_page(_('Connecting...'),
                                  _('Connecting to server...'))
         domain = self.get_page('signup').get_server()
@@ -308,7 +299,7 @@ class AccountWizard(Assistant):
                                self._on_anonymous_supported)
         self._client.connect()
 
-    def _register_with_server(self, ignore_all_errors=False):
+    def _register_with_server(self, ignore_all_errors: bool = False) -> None:
         self._show_progress_page(_('Connecting...'),
                                  _('Connecting to server...'))
         domain = self.get_page('signup').get_server()
@@ -325,7 +316,7 @@ class AccountWizard(Assistant):
 
         self._client.connect()
 
-    def _on_login_successful(self, client, _signal_name):
+    def _on_login_successful(self, client: Client, _signal_name: str) -> None:
         account = self._generate_account_name(client.domain)
         proxy_name = None
         if client.proxy is not None:
@@ -340,11 +331,14 @@ class AccountWizard(Assistant):
         self.get_page('success').set_account(account)
         self.show_page('success', Gtk.StackTransitionType.SLIDE_LEFT)
 
-    def _on_connected(self, client, _signal_name):
+    def _on_connected(self, client: Client, _signal_name: str) -> None:
         client.get_module('Register').request_register_form(
             callback=self._on_register_form)
 
-    def _on_anonymous_supported(self, client, _signal_name):
+    def _on_anonymous_supported(self,
+                                client: Client,
+                                _signal_name: str
+                                ) -> None:
         account = self._generate_account_name(client.domain)
         proxy_name = None
         if client.proxy is not None:
@@ -360,7 +354,7 @@ class AccountWizard(Assistant):
         self.get_page('success').set_account(account)
         self.show_page('success', Gtk.StackTransitionType.SLIDE_LEFT)
 
-    def _on_disconnected(self, client, _signal_name):
+    def _on_disconnected(self, client: Client, _signal_name: str) -> None:
         domain, error, text = client.get_error()
         if domain == StreamError.SASL:
             if error == 'anonymous-not-supported':
@@ -392,13 +386,19 @@ class AccountWizard(Assistant):
             # by the server. This ignores all stream errors if we already
             # know that we succeeded.
             if self.get_current_page() != 'success':
-                self._show_error_page(_('Error'), _('Error'), error)
+                self._show_error_page(_('Error'), _('Error'), text or error)
+
+        else:
+            self._show_error_page(_('Error'), _('Error'), text or error)
 
         self.get_page('form').remove_form()
         self._client.destroy()
         self._client = None
 
-    def _on_connection_failed(self, _client, _signal_name):
+    def _on_connection_failed(self,
+                              _client: Client,
+                              _signal_name: str
+                              ) -> None:
         self._show_error_page(_('Connection failed'),
                               _('Connection failed'),
                               _('Gajim was not able to reach the server. '
@@ -406,31 +406,32 @@ class AccountWizard(Assistant):
         self._client.destroy()
         self._client = None
 
-    def _show_error_page(self, title, heading, text):
+    def _show_error_page(self, title: str, heading: str, text: str) -> None:
         self.get_page('error').set_title(title)
         self.get_page('error').set_heading(heading)
         self.get_page('error').set_text(text or '')
         self.show_page('error', Gtk.StackTransitionType.SLIDE_LEFT)
 
-    def _show_progress_page(self, title, text):
+    def _show_progress_page(self, title: str, text: str) -> None:
         self._progress.set_title(title)
         self._progress.set_text(text)
         self.show_page('progress', Gtk.StackTransitionType.SLIDE_LEFT)
 
     @staticmethod
-    def _generate_account_name(domain):
+    def _generate_account_name(domain: str) -> str:
         i = 1
-        while domain in app.config.get_per('accounts'):
+        while domain in app.settings.get_accounts():
             domain = domain + str(i)
             i += 1
         return domain
 
-    @ensure_not_destroyed
-    def _on_register_form(self, result):
-        if is_error_result(result):
+    def _on_register_form(self, task: Task) -> None:
+        try:
+            result = task.finish()
+        except (StanzaError, MalformedStanzaError) as error:
             self._show_error_page(_('Error'),
                                   _('Error'),
-                                  result.get_text())
+                                  error.get_text())
             self._disconnect()
             return
 
@@ -454,20 +455,42 @@ class AccountWizard(Assistant):
 
         self.show_page('form', Gtk.StackTransitionType.SLIDE_LEFT)
 
-    def _submit_form(self):
+    def _submit_form(self) -> None:
         self.get_page('progress').set_text(_('Account is being created'))
         self.show_page('progress', Gtk.StackTransitionType.SLIDE_LEFT)
 
         form = self.get_page('form').get_submit_form()
         self._client.get_module('Register').submit_register_form(
-            None,
             form,
             callback=self._on_register_result)
 
-    @ensure_not_destroyed
-    def _on_register_result(self, result):
-        if is_error_result(result):
-            self._on_register_error(result)
+    def _on_register_result(self, task: Task) -> None:
+        try:
+            task.finish()
+        except RegisterStanzaError as error:
+            self._set_error_text(error)
+            if error.type != 'modify':
+                self.get_page('form').remove_form()
+                self._disconnect()
+                return
+
+            register_data = error.get_data()
+            form = register_data.form
+            if register_data.form is None:
+                form = register_data.fields_form
+
+            if form is not None:
+                self.get_page('form').add_form(form)
+
+            else:
+                self.get_page('form').remove_form()
+                self._disconnect()
+            return
+
+        except (StanzaError, MalformedStanzaError) as error:
+            self._set_error_text(error)
+            self.get_page('form').remove_form()
+            self._disconnect()
             return
 
         username, password = self.get_page('form').get_credentials()
@@ -489,41 +512,31 @@ class AccountWizard(Assistant):
         self.get_page('form').remove_form()
         self._disconnect()
 
-    def _on_register_error(self, result):
-        error_text = result.get_text()
+    def _set_error_text(self,
+                        error: Union[StanzaError,
+                                     RegisterStanzaError,
+                                     MalformedStanzaError]
+                        ) -> None:
+        error_text = error.get_text()
         if not error_text:
             error_text = _('The server rejected the registration '
                            'without an error message')
         self._show_error_page(_('Error'), _('Error'), error_text)
 
-        register_data = result.get_data()
-        if register_data is None:
-            # IQ error did not have the payload included
-            self.get_page('form').remove_form()
-            self._disconnect()
-            return
-
-        form = register_data.form
-        if register_data.form is None:
-            form = register_data.fields_form
-
-        if form is not None:
-            self.get_page('form').add_form(form)
-
-        else:
-            self.get_page('form').remove_form()
-            self._disconnect()
-
-    def _on_destroy(self, *args):
+    def _on_destroy(self, *args: Any) -> None:
         self._disconnect()
         self._destroyed = True
 
 
 class Login(Page):
-    def __init__(self, button_callback):
+
+    __gsignals__ = {
+        'clicked': (GObject.SignalFlags.RUN_LAST, None, (str,)),
+    }
+
+    def __init__(self) -> None:
         Page.__init__(self)
         self.title = _('Add Account')
-        self._button_callback = button_callback
 
         self._ui = get_builder('account_wizard.ui')
         self._ui.log_in_address_entry.connect(
@@ -542,20 +555,19 @@ class Login(Page):
         self.pack_start(self._ui.login_box, True, True, 0)
         self.show_all()
 
-    def focus(self):
+    def focus(self) -> None:
         self._ui.log_in_address_entry.grab_focus()
 
-    def _on_login(self, *args):
-        self._button_callback(self.get_toplevel(), 'login')
+    def _on_login(self, *args: Any) -> None:
+        self.emit('clicked', 'login')
 
-    def _on_signup(self, *args):
-        self._button_callback(self.get_toplevel(), 'signup')
+    def _on_signup(self, *args: Any) -> None:
+        self.emit('clicked', 'signup')
 
-    def _create_server_completion(self):
+    def _create_server_completion(self) -> None:
         # Parse servers.json
-        server_file_path = os.path.join(
-            configpaths.get('DATA'), 'other', 'servers.json')
-        self._servers = helpers.load_json(server_file_path, 'servers', [])
+        file_path = configpaths.get('DATA') / 'other' / 'servers.json'
+        self._servers = helpers.load_json(file_path, default=[])
 
         # Create a separate model for the address entry, because it will
         # be updated with our localpart@
@@ -564,11 +576,11 @@ class Login(Page):
             address_model.append((server,))
         self._ui.log_in_address_entry.get_completion().set_model(address_model)
 
-    def _on_address_changed(self, entry):
+    def _on_address_changed(self, entry: Gtk.Entry) -> None:
         self._update_completion(entry)
         self._set_complete()
 
-    def _update_completion(self, entry):
+    def _update_completion(self, entry: Gtk.Entry) -> None:
         text = entry.get_text()
         if '@' not in text:
             self._show_icon(False)
@@ -579,28 +591,28 @@ class Login(Page):
         model.clear()
 
         for server in self._servers:
-            model.append(['%s@%s' % (text, server)])
+            model.append([f'{text}@{server}'])
 
-    def _show_icon(self, show):
+    def _show_icon(self, show: bool) -> None:
         icon = 'dialog-warning-symbolic' if show else None
         self._ui.log_in_address_entry.set_icon_from_icon_name(
             Gtk.EntryIconPosition.SECONDARY, icon)
 
-    def _on_address_entry_activate(self, _widget):
+    def _on_address_entry_activate(self, _entry: Gtk.Entry) -> None:
         GLib.idle_add(self._ui.log_in_password_entry.grab_focus)
 
-    def _on_password_entry_activate(self, _widget):
+    def _on_password_entry_activate(self, _entry: Gtk.Entry) -> None:
         if self._ui.log_in_button.get_sensitive():
             self._ui.log_in_button.activate()
 
-    def _validate_jid(self, address):
+    def _validate_jid(self, address: str) -> bool:
         if not address:
             self._show_icon(False)
             return False
 
         try:
             jid = validate_jid(address, type_='bare')
-            if jid.getResource():
+            if jid.resource:
                 raise ValueError
         except ValueError:
             self._show_icon(True)
@@ -611,30 +623,25 @@ class Login(Page):
         self._show_icon(False)
         return True
 
-    def _set_complete(self, *args):
+    def _set_complete(self, *args: Any) -> None:
         address = self._validate_jid(self._ui.log_in_address_entry.get_text())
         password = self._ui.log_in_password_entry.get_text()
         self._ui.log_in_button.set_sensitive(address and password)
 
-    def is_advanced(self):
+    def is_advanced(self) -> bool:
         return self._ui.login_advanced_checkbutton.get_active()
 
-    def get_credentials(self):
+    def get_credentials(self) -> Tuple[str, str]:
         data = (self._ui.log_in_address_entry.get_text(),
                 self._ui.log_in_password_entry.get_text())
         return data
 
 
 class Signup(Page):
-    def __init__(self):
+    def __init__(self) -> None:
         Page.__init__(self)
-        self.complete = False
-        self.title = _('Create New Account')
-
-        # Parse servers.json
-        server_file_path = os.path.join(
-            configpaths.get('DATA'), 'other', 'servers.json')
-        self._servers = helpers.load_json(server_file_path, 'servers', [])
+        self.complete: bool = False
+        self.title: str = _('Create New Account')
 
         self._ui = get_builder('account_wizard.ui')
         self._ui.server_comboboxtext_sign_up_entry.set_activates_default(True)
@@ -653,18 +660,17 @@ class Signup(Page):
 
         self.show_all()
 
-    def focus(self):
+    def focus(self) -> None:
         self._ui.server_comboboxtext_sign_up_entry.grab_focus()
 
-    def _create_server_completion(self):
+    def _create_server_completion(self) -> None:
         # Parse servers.json
-        server_file_path = os.path.join(
-            configpaths.get('DATA'), 'other', 'servers.json')
-        self._servers = helpers.load_json(server_file_path, 'servers', [])
+        file_path = configpaths.get('DATA') / 'other' / 'servers.json'
+        servers = helpers.load_json(file_path, default=[])
 
         # Create servers_model for comboboxes and entries
         servers_model = Gtk.ListStore(str)
-        for server in self._servers:
+        for server in servers:
             servers_model.append((server,))
 
         # Sign up combobox and entry
@@ -672,13 +678,13 @@ class Signup(Page):
         self._ui.server_comboboxtext_sign_up_entry.get_completion().set_model(
             servers_model)
 
-    def _on_visit_server(self, _widget):
+    def _on_visit_server(self, _button: Gtk.Button) -> int:
         server = self._ui.server_comboboxtext_sign_up_entry.get_text().strip()
-        server = 'https://' + server
+        server = f'https://{server}'
         open_uri(server)
         return Gdk.EVENT_STOP
 
-    def _set_complete(self, *args):
+    def _set_complete(self, *args: Any) -> None:
         try:
             self.get_server()
         except Exception:
@@ -688,35 +694,42 @@ class Signup(Page):
             self.complete = True
             self._ui.visit_server_button.set_visible(True)
 
-        self.get_toplevel().update_page_complete()
+        self.update_page_complete()
 
-    def is_anonymous(self):
+    def is_anonymous(self) -> bool:
         return self._ui.sign_up_anonymously.get_active()
 
-    def is_advanced(self):
+    def is_advanced(self) -> bool:
         return self._ui.sign_up_advanced_checkbutton.get_active()
 
-    def get_server(self):
+    def get_server(self) -> str:
         return validate_domainpart(
             self._ui.server_comboboxtext_sign_up_entry.get_text())
 
     @staticmethod
-    def _on_activate_link(_label, uri):
+    def _on_activate_link(_label: Gtk.Label, uri: str) -> int:
         # We have to use this, because the default GTK handler
         # is not cross-platform compatible
         open_uri(uri)
         return Gdk.EVENT_STOP
 
+    def get_visible_buttons(self) -> List[str]:
+        return ['back', 'signup']
+
+    def get_default_button(self) -> str:
+        return 'signup'
+
 
 class AdvancedSettings(Page):
-    def __init__(self):
+    def __init__(self) -> None:
         Page.__init__(self)
-        self.title = _('Advanced settings')
-        self.complete = False
+        self.title: str = _('Advanced settings')
+        self.complete: bool = False
 
         self._ui = get_builder('account_wizard.ui')
         self._ui.manage_proxies_button.connect('clicked',
                                                self._on_proxy_manager)
+        self._ui.proxies_combobox.connect('changed', self._set_complete)
         self._ui.custom_host_entry.connect('changed', self._set_complete)
         self._ui.custom_port_entry.connect('changed', self._set_complete)
         self.pack_start(self._ui.advanced_grid, True, True, 0)
@@ -724,23 +737,25 @@ class AdvancedSettings(Page):
         self.show_all()
 
     @staticmethod
-    def _on_proxy_manager(_widget):
+    def _on_proxy_manager(_button: Gtk.Button) -> None:
         app.app.activate_action('manage-proxies')
 
-    def update_proxy_list(self):
+    def update_proxy_list(self) -> None:
         model = Gtk.ListStore(str)
         self._ui.proxies_combobox.set_model(model)
-        proxies = app.config.get_per('proxies')
+        proxies = app.settings.get_proxies()
         proxies.insert(0, _('No Proxy'))
         for proxy in proxies:
             model.append([proxy])
         self._ui.proxies_combobox.set_active(0)
 
-    def get_proxy(self):
+    def get_proxy(self) -> str:
         active = self._ui.proxies_combobox.get_active()
         return self._ui.proxies_combobox.get_model()[active][0]
 
-    def get_custom_host(self):
+    def get_custom_host(self) -> Optional[Tuple[str,
+                                                ConnectionProtocol,
+                                                ConnectionType]]:
         host = self._ui.custom_host_entry.get_text()
         port = self._ui.custom_port_entry.get_text()
         if not host or not port:
@@ -752,22 +767,25 @@ class AdvancedSettings(Page):
         if host.startswith('ws://') or host.startswith('wss://'):
             protocol = ConnectionProtocol.WEBSOCKET
 
-        return ('%s:%s' % (host, port),
-                protocol,
-                ConnectionType(con_type))
+        return (f'{host}:{port}', protocol, ConnectionType(con_type))
 
-    def _show_host_icon(self, show):
+    def _show_host_icon(self, show: bool) -> None:
         icon = 'dialog-warning-symbolic' if show else None
         self._ui.custom_host_entry.set_icon_from_icon_name(
             Gtk.EntryIconPosition.SECONDARY, icon)
 
-    def _show_port_icon(self, show):
+    def _show_port_icon(self, show: bool) -> None:
         icon = 'dialog-warning-symbolic' if show else None
         self._ui.custom_port_entry.set_icon_from_icon_name(
             Gtk.EntryIconPosition.SECONDARY, icon)
 
-    def _validate_host(self):
+    def _validate_host(self) -> bool:
         host = self._ui.custom_host_entry.get_text()
+        if host.startswith('ws://') or host.startswith('wss://'):
+            # We have no method for validating websocket URIs
+            self._show_host_icon(False)
+            return True
+
         try:
             validate_domainpart(host)
         except Exception:
@@ -779,7 +797,7 @@ class AdvancedSettings(Page):
         self._show_host_icon(False)
         return True
 
-    def _validate_port(self):
+    def _validate_port(self) -> bool:
         port = self._ui.custom_port_entry.get_text()
         if not port:
             self._show_port_icon(False)
@@ -803,19 +821,39 @@ class AdvancedSettings(Page):
         self._show_port_icon(False)
         return True
 
-    def _set_complete(self, *args):
-        port_valid = self._validate_port()
-        host_valid = self._validate_host()
-        self.complete = port_valid and host_valid
-        self.get_toplevel().update_page_complete()
+    def _is_custom_host_set(self) -> bool:
+        host = bool(self._ui.custom_host_entry.get_text())
+        port = bool(self._ui.custom_port_entry.get_text())
+        return host or port
+
+    def _is_proxy_set(self) -> bool:
+        return self._ui.proxies_combobox.get_active() != 0
+
+    def _set_complete(self, *args: Any) -> None:
+        self.complete = False
+        if self._is_proxy_set():
+            self.complete = True
+
+        if self._is_custom_host_set():
+            port_valid = self._validate_port()
+            host_valid = self._validate_host()
+            self.complete = port_valid and host_valid
+
+        self.update_page_complete()
+
+    def get_visible_buttons(self) -> List[str]:
+        return ['back', self.get_toplevel().get_currenct_method()]
+
+    def get_default_button(self) -> str:
+        return self.get_toplevel().get_currenct_method()
 
 
 class SecurityWarning(Page):
-    def __init__(self):
+    def __init__(self) -> None:
         Page.__init__(self)
-        self.title = _('Security Warning')
-        self._cert = None
-        self._domain = None
+        self.title: str = _('Security Warning')
+        self._cert: Optional[Gio.TlsCertificate] = None
+        self._domain: Optional[str] = None
 
         self._ui = get_builder('account_wizard.ui')
         self.pack_start(self._ui.security_warning_box, True, True, 0)
@@ -823,10 +861,14 @@ class SecurityWarning(Page):
         self.show_all()
 
     @property
-    def cert(self):
+    def cert(self) -> Optional[Gio.TlsCertificate]:
         return self._cert
 
-    def set_warning(self, domain, cert, errors):
+    def set_warning(self,
+                    domain: str,
+                    cert: Gio.TlsCertificate,
+                    errors: List[Gio.TlsCertificateFlags]
+                    ) -> None:
         # Clear list
         self._cert = cert
         self._domain = domain
@@ -851,24 +893,30 @@ class SecurityWarning(Page):
         self._ui.trust_cert_checkbutton.set_visible(
             Gio.TlsCertificateFlags.UNKNOWN_CA in errors)
 
-    def _on_view_cert(self, _button):
+    def _on_view_cert(self, _button: Gtk.Button) -> None:
         open_window('CertificateDialog',
                     account=self._domain,
                     transient_for=self.get_toplevel(),
                     cert=self._cert)
 
     @property
-    def trust_certificate(self):
+    def trust_certificate(self) -> bool:
         return self._ui.trust_cert_checkbutton.get_active()
 
+    def get_visible_buttons(self) -> List[str]:
+        return ['back', self.get_toplevel().get_currenct_method()]
+
+    def get_default_button(self) -> str:
+        return 'back'
+
 
 class Form(Page):
-    def __init__(self):
+    def __init__(self) -> None:
         Page.__init__(self)
         self.set_valign(Gtk.Align.FILL)
-        self.complete = False
-        self.title = _('Create Account')
-        self._current_form = None
+        self.complete: bool = False
+        self.title: str = _('Create Account')
+        self._current_form: Optional[Any] = None
 
         heading = Gtk.Label(label=_('Create Account'))
         heading.get_style_context().add_class('large-header')
@@ -881,14 +929,14 @@ class Form(Page):
         self.show_all()
 
     @property
-    def has_form(self):
+    def has_form(self) -> bool:
         return self._current_form is not None
 
-    def _on_is_valid(self, _widget, is_valid):
+    def _on_is_valid(self, _widget: DataFormWidget, is_valid: bool) -> None:
         self.complete = is_valid
-        self.get_toplevel().update_page_complete()
+        self.update_page_complete()
 
-    def add_form(self, form):
+    def add_form(self, form: Any) -> None:
         self.remove_form()
 
         options = {'hide-fallback-fields': True,
@@ -899,14 +947,14 @@ class Form(Page):
         self.pack_start(self._current_form, True, True, 0)
         self._current_form.show_all()
 
-    def get_credentials(self):
+    def get_credentials(self) -> Tuple[str, str]:
         return (self._current_form.get_form()['username'].value,
                 self._current_form.get_form()['password'].value)
 
-    def get_submit_form(self):
+    def get_submit_form(self) -> Any:
         return self._current_form.get_submit_form()
 
-    def remove_form(self):
+    def remove_form(self) -> None:
         if self._current_form is None:
             return
 
@@ -914,42 +962,115 @@ class Form(Page):
         self._current_form.destroy()
         self._current_form = None
 
-    def focus(self):
+    def focus(self) -> None:
         self._current_form.focus_first_entry()
 
+    def get_visible_buttons(self) -> List[str]:
+        return ['back', 'signup']
+
+    def get_default_button(self) -> str:
+        return 'signup'
+
 
 class Redirect(Page):
-    def __init__(self):
+    def __init__(self) -> None:
         Page.__init__(self)
-        self.title = _('Redirect')
-        self._link = None
+        self.title: str = _('Redirect')
+        self._link: Optional[str] = None
 
         self._ui = get_builder('account_wizard.ui')
         self.pack_start(self._ui.redirect_box, True, True, 0)
         self._ui.link_button.connect('clicked', self._on_link_button)
         self.show_all()
 
-    def set_redirect(self, link, instructions):
+    def set_redirect(self, link: str, instructions: str) -> None:
         if instructions is None:
             instructions = _('Register on the Website')
         self._ui.instructions.set_text(instructions)
         self._link = link
 
-    def _on_link_button(self, _button):
+    def _on_link_button(self, _button:Gtk.Button) -> None:
         open_uri(self._link)
 
+    def get_visible_buttons(self) -> List[str]:
+        return ['back']
+
 
 class Success(SuccessPage):
-    def __init__(self):
+    def __init__(self) -> None:
         SuccessPage.__init__(self)
         self.set_title(_('Account Added'))
         self.set_heading(_('Account has been added successfully'))
 
-        self._account = None
+        self._account: Optional[str] = None
+        self._our_jid: Optional[str] = None
+        self._label: Optional[str] = None
+        self._color: Optional[str] = None
 
-    def set_account(self, account):
+        self._ui = get_builder('account_wizard.ui')
+        self.pack_start(self._ui.account_label_box, True, True, 0)
+
+        self._provider = self._add_css_provider()
+
+        self._ui.account_name_entry.connect('changed', self._on_name_changed)
+        self._ui.account_color_button.connect('color-set', self._on_color_set)
+
+        self.show_all()
+
+    def set_account(self, account: str) -> None:
         self._account = account
+        self._our_jid = app.get_jid_from_account(account)
+        self._ui.badge_preview.set_text(self._our_jid)
+        rgba = Gdk.RGBA()
+        rgba.parse(get_color_for_account(self._our_jid))
+        self._ui.account_color_button.set_rgba(rgba)
+        self._color = rgba.to_string()
+        self._set_badge_color(self._color)
 
     @property
-    def account(self):
+    def account(self) -> Optional[str]:
         return self._account
+
+    def _add_css_provider(self) -> Gtk.CssProvider:
+        context = self._ui.badge_preview.get_style_context()
+        provider = Gtk.CssProvider()
+        context.add_provider(provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)
+        return provider
+
+    def _on_name_changed(self, entry: Gtk.Entry):
+        self._label = entry.get_text()
+        self._ui.badge_preview.set_text(self._label or self._our_jid)
+        self._save_config()
+
+    def _on_color_set(self, button: Gtk.ColorButton):
+        rgba = button.get_rgba()
+        self._color = rgba.to_string()
+        self._set_badge_color(self._color)
+        self._save_config()
+
+    def _set_badge_color(self, color: str) -> None:
+        css = '.badge { background-color: %s; font-size: 100%%; }' % color
+        self._provider.load_from_data(bytes(css.encode()))
+
+    def _save_config(self) -> None:
+        app.settings.set_account_setting(
+            self._account, 'account_color', self._color)
+        if self._label:
+            app.settings.set_account_setting(
+                self._account, 'account_label', self._label)
+        app.css_config.refresh()
+        window = get_app_window('AccountsWindow')
+        if window is not None:
+            window.update_account_label(self._account)
+
+    def get_visible_buttons(self) -> List[str]:
+        return ['connect']
+
+
+class Error(ErrorPage):
+    def __init__(self) -> None:
+        ErrorPage.__init__(self)
+        self.set_heading(_('An error occurred during account creation'))
+
+    def get_visible_buttons(self) -> List[str]:
+        return ['back']
