@@ -19,14 +19,14 @@ from gi.repository import Pango
 
 from nbxmpp.modules.dataforms import extend_form
 
-from gajim.gtkgui_helpers import scale_pixbuf_from_data
-
 from gajim.common import app
 from gajim.common.i18n import _
 from gajim.common.helpers import open_uri
 
-from gajim.gtk.util import MultiLineLabel
-from gajim.gtk.util import MaxWidthComboBoxText
+from .util import MultiLineLabel
+from .util import MaxWidthComboBoxText
+from .util import make_href_markup
+from .util import scale_pixbuf_from_data
 
 
 # Options
@@ -59,6 +59,7 @@ class DataFormWidget(Gtk.ScrolledWindow):
 
         self._form_node = form_node
         self._form_grid = FormGrid(form_node, options)
+        self._original_form_hash = self.get_form_hash()
 
         self.add(self._form_grid)
 
@@ -80,6 +81,15 @@ class DataFormWidget(Gtk.ScrolledWindow):
         self._form_node.type_ = 'submit'
         return self._form_node
 
+    def get_form_hash(self):
+        return hash(str(self._form_node))
+
+    def was_modified(self):
+        return self._original_form_hash != self.get_form_hash()
+
+    def reset_form_hash(self):
+        self._original_form_hash = self.get_form_hash()
+
     def focus_first_entry(self):
         for row in range(0, self._form_grid.row_count):
             widget = self._form_grid.get_child_at(1, row)
@@ -219,7 +229,8 @@ class Title:
 
 class Instructions:
     def __init__(self, instructions):
-        self._label = Gtk.Label(label=instructions)
+        self._label = Gtk.Label()
+        self._label.set_markup(make_href_markup(instructions))
         self._label.set_line_wrap(True)
         self._label.set_line_wrap_mode(Pango.WrapMode.WORD)
         self._label.set_justify(Gtk.Justification.CENTER)
@@ -304,7 +315,7 @@ class Field:
             self._form_grid.validate(is_valid)
             self._validate_source_id = None
 
-        self._validate_source_id = GLib.timeout_add(500, _start_validation)
+        self._validate_source_id = GLib.timeout_add(200, _start_validation)
 
 
 class BooleanField(Field):
@@ -323,13 +334,14 @@ class BooleanField(Field):
 
     def _toggled(self, _widget):
         self._field.value = self._widget.get_active()
+        self._validate()
 
 
 class FixedField(Field):
     def __init__(self, field, form_grid, options):
         Field.__init__(self, field, form_grid, options)
 
-        self._label.set_text(field.value)
+        self._label.set_markup(make_href_markup(field.value))
 
         # If the value is more than 40 chars it proabably isn’t
         # meant as a section header
@@ -361,6 +373,7 @@ class ListSingleField(Field):
 
     def _changed(self, widget):
         self._field.value = widget.get_active_id()
+        self._validate()
 
 
 class ListMultiField(Field):
@@ -374,6 +387,7 @@ class ListMultiField(Field):
         self._widget.set_propagate_natural_height(True)
         self._widget.set_min_content_height(100)
         self._widget.set_max_content_height(300)
+        self._widget.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         self._widget.add(self._treeview)
 
     def validate(self):
@@ -387,10 +401,13 @@ class ListMutliTreeView(Gtk.TreeView):
         self._field = field
         self._multi_field = multi_field
 
-        self._store = Gtk.ListStore(str, str, bool)
+        # label, value, tooltip, toggled
+        self._store = Gtk.ListStore(str, str, str, bool)
 
         col = Gtk.TreeViewColumn()
         cell = Gtk.CellRendererText()
+        cell.set_property('ellipsize', Pango.EllipsizeMode.END)
+        cell.set_property('width-chars', 40)
         col.pack_start(cell, True)
         col.set_attributes(cell, text=0)
         self.append_column(col)
@@ -402,31 +419,36 @@ class ListMutliTreeView(Gtk.TreeView):
         cell.set_property('xpad', 10)
         cell.connect('toggled', self._toggled)
         col.pack_start(cell, True)
-        col.set_attributes(cell, active=2)
+        col.set_attributes(cell, active=3)
         self.append_column(col)
 
         self.set_headers_visible(False)
 
         for option in field.options:
-            # option = (label, value)
+            label, value = option
             self._store.append(
-                [*option, option[1] in field.values])
+                [label, value, label, value in field.values])
+
+        labels_over_max_width = map(lambda x: len(x) > 40,
+                                    [option[0] for option in field.options])
+        if any(labels_over_max_width):
+            self.set_tooltip_column(2)
 
         self.set_model(self._store)
 
     def _toggled(self, _renderer, path):
         iter_ = self._store.get_iter(path)
-        current_value = self._store[iter_][2]
-        self._store.set_value(iter_, 2, not current_value)
+        current_value = self._store[iter_][3]
+        self._store.set_value(iter_, 3, not current_value)
         self._set_values()
         self._multi_field.validate()
 
     def _set_values(self):
         values = []
         for row in self.get_model():
-            if not row[2]:
+            if not row[3]:
                 continue
-            values.append(row[1])
+            values.append(row[2])
         self._field.values = values
 
 
@@ -529,6 +551,7 @@ class TextSingleField(Field):
         if self.read_only:
             self._widget = Gtk.Label(label=field.value)
             self._widget.set_xalign(0)
+            self._widget.set_selectable(True)
         else:
             self._widget = Gtk.Entry()
             self._widget.set_text(field.value)
