@@ -12,6 +12,9 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Optional
+from typing import Union
+
 import logging
 import random
 
@@ -19,24 +22,31 @@ from gi.repository import Gtk
 from gi.repository import Gdk
 
 from nbxmpp.errors import StanzaError
+from nbxmpp.task import Task
 
 from gajim.common import app
+from gajim.common import ged
 from gajim.common.const import MUC_CREATION_EXAMPLES
 from gajim.common.const import MUC_DISCO_ERRORS
-from gajim.common.i18n import _
+from gajim.common.events import AccountConnected
+from gajim.common.events import AccountDisonnected
+from gajim.common.ged import EventHelper
+from gajim.common.helpers import get_random_muc_localpart
 from gajim.common.helpers import validate_jid
 from gajim.common.helpers import to_user_string
+from gajim.common.i18n import _
 
 from .dialogs import ErrorDialog
-from .util import get_builder
+from .builder import get_builder
 from .util import ensure_not_destroyed
 
 log = logging.getLogger('gajim.gui.groupchat_creation')
 
 
-class CreateGroupchatWindow(Gtk.ApplicationWindow):
-    def __init__(self, account):
+class CreateGroupchatWindow(Gtk.ApplicationWindow, EventHelper):
+    def __init__(self, account: Optional[str]) -> None:
         Gtk.ApplicationWindow.__init__(self)
+        EventHelper.__init__(self)
         self.set_name('CreateGroupchat')
         self.set_application(app.app)
         self.set_position(Gtk.WindowPosition.CENTER)
@@ -47,44 +57,63 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow):
         self.set_title(_('Create Group Chat'))
 
         self._ui = get_builder('groupchat_creation.ui')
-        self.add(self._ui.create_group_chat)
+        self.add(self._ui.stack)
 
-        self._destroyed = False
-
-        self._account = self._fill_account_combo(account)
+        self._account = account
+        self._destroyed: bool = False
 
         self._create_entry_completion()
-        self._fill_placeholders()
 
         self._ui.connect_signals(self)
-        self.connect('key-press-event', self._on_key_press_event)
+        self.connect('key-press-event', self._on_key_press)
         self.connect('destroy', self._on_destroy)
 
+        self.register_events([
+            ('account-connected', ged.GUI2, self._on_account_state),
+            ('account-disconnected', ged.GUI2, self._on_account_state)
+        ])
+
         self.show_all()
+
+        if app.get_number_of_connected_accounts() == 0:
+            # This can happen under rare circumstances
+            self._ui.stack.set_visible_child_name('no-connection')
+            return
+
+        self._update_accounts(account)
+
         self.set_focus(self._ui.address_entry)
 
-    def _get_muc_service_jid(self):
-        con = app.connections[self._account]
-        return str(con.get_module('MUC').service_jid or 'muc.example.com')
+    def _on_account_state(self,
+                          _event: Union[AccountConnected, AccountDisonnected]
+                          ) -> None:
+        any_account_connected = app.get_number_of_connected_accounts() > 0
+        if any_account_connected:
+            self._ui.stack.set_visible_child_name('create')
+            self._update_accounts()
+        else:
+            self._ui.stack.set_visible_child_name('no-connection')
 
-    def _fill_account_combo(self, account):
+    def _update_accounts(self, account: Optional[str] = None) -> None:
         accounts = app.get_enabled_accounts_with_labels(connected_only=True)
         account_liststore = self._ui.account_combo.get_model()
+        assert isinstance(account_liststore, Gtk.ListStore)
+        account_liststore.clear()
+
         for acc in accounts:
             account_liststore.append(acc)
 
-        # Hide account combobox if there is only one account
-        if len(accounts) == 1:
-            self._ui.account_combo.hide()
-            self._ui.account_label.hide()
+        self._ui.account_combo.set_visible(len(accounts) != 1)
+        self._ui.account_label.set_visible(len(accounts) != 1)
 
         if account is None:
             account = accounts[0][0]
 
         self._ui.account_combo.set_active_id(account)
-        return account
+        self._account = account
+        self._fill_placeholders()
 
-    def _create_entry_completion(self):
+    def _create_entry_completion(self) -> None:
         entry_completion = Gtk.EntryCompletion()
         model = Gtk.ListStore(str)
         entry_completion.set_model(model)
@@ -95,37 +124,37 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow):
 
         self._ui.address_entry.set_completion(entry_completion)
 
-    def _fill_placeholders(self):
+    def _fill_placeholders(self) -> None:
         placeholder = random.choice(MUC_CREATION_EXAMPLES)
         server = self._get_muc_service_jid()
 
         self._ui.name_entry.set_placeholder_text(
-            placeholder[0] + _(' (optional)...'))
+            _('e.g. %s') % placeholder[0])
         self._ui.description_entry.set_placeholder_text(
-            placeholder[1] + _(' (optional)...'))
+            _('e.g. %s' % placeholder[1]))
         self._ui.address_entry.set_placeholder_text(
-            '%s@%s' % (placeholder[2], server))
+            f'{placeholder[2]}@{server}')
 
-    def _on_key_press_event(self, _widget, event):
+    def _get_muc_service_jid(self) -> str:
+        assert self._account is not None
+        client = app.get_client(self._account)
+        return str(client.get_module('MUC').service_jid or 'muc.example.com')
+
+    def _on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> None:
         if event.keyval == Gdk.KEY_Escape:
             self.destroy()
 
-    def _on_account_combo_changed(self, combo):
+    def _on_account_combo_changed(self, combo: Gtk.ComboBox) -> None:
         self._account = combo.get_active_id()
+        if self._account is None:
+            model = combo.get_model()
+            iter_ = model.get_iter_first()
+            if not iter_:
+                return
+            self._account = model[iter_][0]
         self._fill_placeholders()
 
-    def _update_entry_completion(self, entry, text):
-        text = entry.get_text()
-        if '@' in text:
-            text = text.split('@', 1)[0]
-
-        model = entry.get_completion().get_model()
-        model.clear()
-
-        server = self._get_muc_service_jid()
-        model.append(['%s@%s' % (text, server)])
-
-    def _validate_jid(self, text):
+    def _validate_jid(self, text: str) -> None:
         if not text:
             self._set_warning_icon(False)
             self._ui.create_button.set_sensitive(False)
@@ -142,7 +171,7 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow):
             self._set_warning_icon(False)
             self._ui.create_button.set_sensitive(True)
 
-    def _set_processing_state(self, enabled):
+    def _set_processing_state(self, enabled: bool) -> None:
         if enabled:
             self._ui.spinner.start()
             self._ui.create_button.set_sensitive(False)
@@ -150,58 +179,86 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow):
             self._ui.spinner.stop()
         self._ui.grid.set_sensitive(not enabled)
 
-    def _set_warning_icon(self, enabled):
+    def _set_warning_icon(self, enabled: bool) -> None:
         icon = 'dialog-warning-symbolic' if enabled else None
         self._ui.address_entry.set_icon_from_icon_name(
             Gtk.EntryIconPosition.SECONDARY, icon)
 
-    def _set_warning_tooltip(self, text):
+    def _set_warning_tooltip(self, text: str) -> None:
         self._ui.address_entry.set_icon_tooltip_text(
             Gtk.EntryIconPosition.SECONDARY, text)
 
-    def _set_warning(self, text):
+    def _set_warning(self, text: str) -> None:
         self._set_warning_icon(True)
         self._set_warning_tooltip(text)
         self._ui.create_button.set_sensitive(False)
 
-    def _set_warning_from_error(self, error):
-        condition = error.condition
+    def _set_warning_from_error(self, error: StanzaError) -> None:
+        condition = error.condition or ''
         if condition == 'gone':
             condition = 'already-exists'
         text = MUC_DISCO_ERRORS.get(condition, to_user_string(error))
         self._set_warning(text)
 
-    def _set_warning_from_error_code(self, error_code):
+    def _set_warning_from_error_code(self, error_code: str) -> None:
         self._set_warning(MUC_DISCO_ERRORS[error_code])
 
-    def _on_address_entry_changed(self, entry):
+    def _on_name_entry_changed(self, entry: Gtk.Entry) -> None:
+        if self._ui.private_radio.get_active():
+            return
+
+        name = entry.get_text()
+        server = self._get_muc_service_jid()
+        self._ui.address_entry.set_text(f'{name.lower()}@{server}')
+
+    def _on_address_entry_changed(self, entry: Gtk.Entry) -> None:
         text = entry.get_text()
         self._update_entry_completion(entry, text)
         self._validate_jid(text)
 
-    def _on_address_entry_activate(self, _widget):
-        self._on_create_clicked()
+    def _update_entry_completion(self, entry: Gtk.Entry, text: str) -> None:
+        text = entry.get_text()
+        if '@' in text:
+            text = text.split('@', 1)[0]
 
-    def _on_create_clicked(self, *args):
+        model = entry.get_completion().get_model()
+        assert isinstance(model, Gtk.ListStore)
+        model.clear()
+
+        server = self._get_muc_service_jid()
+        model.append([f'{text}@{server}'])
+
+    def _on_public_private_toggled(self, _radiobutton: Gtk.RadioButton) -> None:
+        is_public = self._ui.public_radio.get_active()
+        self._ui.address_label.set_visible(is_public)
+        self._ui.address_entry.set_visible(is_public)
+
+    def _on_create_clicked(self, _button: Gtk.Button) -> None:
+        assert self._account is not None
         if not app.account_is_available(self._account):
             ErrorDialog(
                 _('Not Connected'),
                 _('You have to be connected to create a group chat.'))
             return
 
-        room_jid = self._ui.address_entry.get_text()
+        if self._ui.public_radio.get_active():
+            room_jid = self._ui.address_entry.get_text()
+        else:
+            server = self._get_muc_service_jid()
+            room_jid = f'{get_random_muc_localpart()}@{server}'
 
         self._set_processing_state(True)
-        con = app.connections[self._account]
-        con.get_module('Discovery').disco_info(
+        client = app.get_client(self._account)
+        client.get_module('Discovery').disco_info(
             room_jid, callback=self._disco_info_received)
 
     @ensure_not_destroyed
-    def _disco_info_received(self, task):
+    def _disco_info_received(self, task: Task) -> None:
         try:
             result = task.finish()
         except StanzaError as error:
             if error.condition == 'item-not-found':
+                assert error.jid is not None
                 self._create_muc(error.jid)
                 return
             self._set_warning_from_error(error)
@@ -212,10 +269,10 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow):
 
         self._set_processing_state(False)
 
-    def _create_muc(self, room_jid):
+    def _create_muc(self, room_jid: str) -> None:
         name = self._ui.name_entry.get_text()
         description = self._ui.description_entry.get_text()
-        is_public = self._ui.public_switch.get_active()
+        is_public = self._ui.public_radio.get_active()
 
         config = {
             # XEP-0045 options
@@ -231,12 +288,15 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow):
         }
 
         # Create new group chat by joining
+        assert self._account is not None
         app.interface.create_groupchat(
             self._account,
             str(room_jid),
-            config=config)
+            config)
 
         self.destroy()
 
-    def _on_destroy(self, *args):
+    def _on_destroy(self, _widget: Gtk.Widget) -> None:
         self._destroyed = True
+        self.unregister_events()
+        app.check_finalize(self)
